import { injectable, inject } from "inversify";
import { CommandContribution, CommandRegistry, MenuContribution, MenuModelRegistry, MessageService } from "@theia/core/lib/common";
import { TerminalService } from '@theia/terminal/lib/browser/base/terminal-service';
import { TerminalWidget } from '@theia/terminal/lib/browser/base/terminal-widget';
import { TerminalWidgetFactoryOptions } from '@theia/terminal/lib/browser/terminal-widget-impl';
import { CommonMenus } from "@theia/core/lib/browser";
import { WindowService } from '@theia/core/lib/browser/window/window-service';
import { MAIN_MENU_BAR } from '@theia/core';
import { FileSystemWatcher } from '@theia/filesystem/lib/browser/filesystem-watcher';
import { FileSystem } from '@theia/filesystem/lib/common';
import URI from '@theia/core/lib/common/uri';
import {FileResource } from '@theia/filesystem/lib/browser/file-resource';
//import { EnvVariablesServer } from '@theia/core/lib/common/env-variables';


 
//import URI from '@theia/core/lib/common/uri';

export const NpmstartExtensionCommand = {
    id: 'NpmstartExtension.command',
    label: "Shows a message"
};
export const NpmbuildExtensionCommand = {
    id: 'NpmbuildExtension.command',
    label: "Shows a message"
};
export const OpenUrlCommandCommand = {
    id: 'OpenUrlCommand.command',
    label: "Shows a message"
};


export namespace RunMenus {
    export const RUN = [...MAIN_MENU_BAR, '7_run'];
    export const RUN_NEW = [...RUN, '1_run'];
    export const BUILD_NEW = [...RUN, '3_run'];	
    export const RUN_TASKS = [...RUN, '2_run'];
	    export const OPEN_NEW = [...RUN, '4_run'];
   
}
@injectable()
export class NpmstartExtensionCommandContribution implements CommandContribution {
     //uri: URI;
    constructor(
        @inject(MessageService) private readonly messageService: MessageService,
		@inject(TerminalService) protected readonly terminalService: TerminalService,
		@inject(WindowService) protected readonly windowService: WindowService,
		@inject(FileSystem) protected readonly fileSystem: FileSystem,
        @inject(FileSystemWatcher) protected readonly watcher: FileSystemWatcher,
		//protected envVariablesServer:EnvVariablesServer
		
    ) { }

    registerCommands(registry: CommandRegistry): void {
        registry.registerCommand(NpmstartExtensionCommand, {
            execute: () => {
				this.messageService.info('Running...');
                this.runSelectedText();
			}
        });
        registry.registerCommand(NpmbuildExtensionCommand, {
            execute: () => {
				this.messageService.info('Building...');
						//this.readFileContents();	
                this.buildSelectedText(); 
                 //this.getMyIp();
				// console.log("IP"+ip);
				//this.envVariablesServer.getValue('HOME').then(value => {
			//this.messageService.info('ENV...'+value);
         //}); 
				//this.windowService.openNewWindow("http://www.programmr.com");
			}
        });	
            registry.registerCommand(OpenUrlCommandCommand, {
            execute: () => {
				this.messageService.info('Opened in new tab...');
                 this.readFileContents();	

			}
        });	
		
    }
	
	async readFileContents(): Promise<void>{
			const uri = new URI("/home/ubuntu/myip.txt");
				 const resource = new FileResource(uri, this.fileSystem, this.watcher);
				let contents = await resource.readContents();
				console.log(contents);
				let myIp = contents.trim();
					console.log("ip=["+myIp+"]");
		       this.windowService.openNewWindow("http://"+myIp+":4200");
	}
	
		getMyIp() {
			  const request = new XMLHttpRequest();
              var that = this;
        request.onreadystatechange = function () {
            if (this.readyState === XMLHttpRequest.DONE) {
                if (this.status === 200) {
                    console.log(this.response);
					const data = JSON.parse(this.response);
					that.windowService.openNewWindow("http://"+data.ip+":4200");
					
                } else {
                    throw(new Error('Could not fetch'));
                }
            }
        };

        request.open('GET', 'http://api.ipify.org/?format=json', true);
        request.send();
		
	}
	
	 async runSelectedText(): Promise<void> {
					    let terminal = this.terminalService.currentTerminal;
				if (!terminal) {
					terminal = <TerminalWidget>await this.terminalService.newTerminal(<TerminalWidgetFactoryOptions>{ created: new Date().toString() });
					await terminal.start();
					this.terminalService.activateTerminal(terminal);
				}
				terminal.sendText("npm start\n");  
		 
	 }
	 async buildSelectedText(): Promise<void> {
					    let terminal = this.terminalService.currentTerminal;
				if (!terminal) {
					terminal = <TerminalWidget>await this.terminalService.newTerminal(<TerminalWidgetFactoryOptions>{ created: new Date().toString() });
					await terminal.start();
					this.terminalService.activateTerminal(terminal);
				}
				terminal.sendText("npm install\n");  
		 
	 }	 
	 
}

@injectable()
export class NpmstartExtensionMenuContribution implements MenuContribution {
	


    registerMenus(menus: MenuModelRegistry): void {
		
		menus.registerSubmenu(RunMenus.RUN, 'Run');
		menus.registerMenuAction(RunMenus.BUILD_NEW, {
            commandId: NpmbuildExtensionCommand.id,
            label: 'Build (npm install)',
            order: '0'
        });			
        menus.registerMenuAction(RunMenus.RUN_NEW, {
            commandId: NpmstartExtensionCommand.id,
            label: 'Run (npm start)',
            order: '1'
        });	
        menus.registerMenuAction(RunMenus.OPEN_NEW, {
            commandId: OpenUrlCommandCommand.id,
            label: 'View Output',
            order: '2'
        });			
		
        menus.registerMenuAction(CommonMenus.EDIT_FIND, {
            commandId: NpmstartExtensionCommand.id,
            label: 'Run (npm start)'
        });
        menus.registerMenuAction(CommonMenus.EDIT_FIND, {
            commandId: NpmbuildExtensionCommand.id,
            label: 'Build (npm install)'
        });		
    }
}